--*************************************
-- Gideros SDK example
-- Juggle a soccer ball
--*************************************
-- Author: Michael Hartlef    (aka MikeHart)
-- Date:   December 12th, 2011
-- Email:  mike@fantomgl.com
--*************************************

-- require the Box2D physic module
require "box2d"

-- Set the scale rate for the physics
local physicsScale = 60
b2.setScale(physicsScale)

-- To have the socccer ball come down again, we set some gravity
--local world = b2.World.new(0, 9.8/(60/physicsScale), true)
local world = b2.World.new(0, 4.0/(60/physicsScale), true)

-- The next function will define a physical edge
function createEdge(x1, y1, x2, y2, name)
	-- Create the body
	local edge = world:createBody({})
	-- Give the body a name
	edge.name = name
	-- Create a shape
	local eshape = b2.EdgeShape.new()
	-- Set one line for the edge
	eshape:set(x1, y1, x2, y2)
	-- Create the fixture
	edge:createFixture({shape = eshape, density = 0, filter={categoryBits = 1, maskBits = 2}})
end

-- Create the top edge
createEdge(0, 2, application:getContentWidth(), 2, "Top")
-- Create the botton edge
createEdge(0, application:getContentHeight()-2, application:getContentWidth(), application:getContentHeight()-2, "Bottom")
-- Create the left edge
createEdge(2, 2, 2, application:getContentHeight()-2, "Left")
-- Create the right edge
createEdge(application:getContentWidth()-2, 2, application:getContentWidth()-2, application:getContentHeight()-2, "Right")

-- Load the background image and add as a child of stage
stage:addChild(Bitmap.new(Texture.new("images/field.png")))

-- Next we define the balls physic body
local ballBody = world:createBody{type = b2.DYNAMIC_BODY, position = {x=application:getContentWidth()/2.0, y=30.0}}
-- Define the ball shape
local ballshape = b2.CircleShape.new(0, 0, 32)
-- Create the fixture
ballBody:createFixture{shape = ballshape, density = 0.7, restitution = 0.6, friction = 0.2, filter = {categoryBits = 2, maskBits = 1}}


-- create a ball bitmap object (Bitmap class inherits from Sprite class)
local soccerball = Bitmap.new(Texture.new("images/ball.png"))
-- add the ball sprite to the stage
stage:addChild(soccerball)

-- Set the reference points of the ball to its center
soccerball:setAnchorPoint(0.5, 0.5)

-- Position the ball finally on the screen
soccerball:setPosition(application:getContentWidth()/2.0, 30.0)

-- Add the physic body to the ball object
soccerball.body = ballBody

-- give the ball a name
ballBody.name = "Ball"

-- Create score and highscore
local score = 0
local hscore = 0

-- Load the font file
local font = Font.new("images/arial.txt", "images/arial.png")
--local font = Font.new("images/Impact.txt", "images/Impact.png")

-- Create a text field for the score
local txtScore = TextField.new(font, "Score: "..score)
txtScore:setPosition(10, 20)
stage:addChild(txtScore)

-- Create a text field for the high score
local txtHScore = TextField.new(font, "HighScore: "..hscore)
txtHScore:setPosition(180, 20)
stage:addChild(txtHScore)


-- The function onTouch deals with the touch of our finger, so we can try to juggle the ball in the air
function onTouch(self, event)
	-- Check if the touch coordinates have hit the object
	if self:hitTestPoint(event.x, event.y) then
		-- Get the objects physic body
		local body = self.body
		-- Get the body's position
		local bx, by = body:getPosition()
		-- Apply some force to the object
		body:applyForce((bx - event.x)*10*(60/physicsScale), (by - event.y)*15*(60/physicsScale), bx, by)
		-- Raise the score and set the corresponding text field
		score = score + 1
		txtScore:setText("Score: "..score)
	end
end

-- The next function will set the objects position and rotation values regarding its physical body values
function soccerball:onEnterFrame(event)
	-- Get the objects physic body
	local body = self.body
	-- Set the objects position to the same like the body
	self:setPosition(body:getPosition())
	-- Set the objects rotation value to the same like the body
	self:setRotation(body:getAngle() * 180 / math.pi)
end

-- The next function will update the world physics
function onEnterFrame(event)
	-- Update the physic world
	world:step(1/physicsScale, 8, 3)
end

local function onBeginContact(event)
	-- you can get the fixtures and bodies in this contact like:
	local fixtureA = event.fixtureA
	local fixtureB = event.fixtureB
	local bodyA = fixtureA:getBody()
	local bodyB = fixtureB:getBody()
	
	--print("begin contact: "..bodyA.name.." "..bodyB.name)
	if bodyA.name == "Bottom" then
		-- Check if the current score is a new highscore
		if score > hscore then
			-- Set the new high score and update the text field
			hscore = score
			txtHScore:setText("HighScore: "..hscore)
		end
		-- Reset the score and update the text field
		score = 0
		txtScore:setText("Score: "..score)
	end
end

-- Add event listener for the stage and each frame
stage:addEventListener(Event.ENTER_FRAME, onEnterFrame)
-- add event listener for the soccer ball and the mouse
soccerball:addEventListener(Event.MOUSE_DOWN, onTouch, soccerball)
-- Add event listener for the soccer ball and each frame
soccerball:addEventListener(Event.ENTER_FRAME, soccerball.onEnterFrame, soccerball)
-- Add event listener for contact handling
world:addEventListener(Event.BEGIN_CONTACT, onBeginContact)

